package com.restlet.client.model;

import com.restlet.client.model.environment.EnvironmentTo;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.tests.ScenarioTo;

import java.io.Serializable;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;

import static java.util.Arrays.asList;

public interface EntityTo extends Serializable {

    Type getType();

    void setType(Type type);

    String getName();

    void setName(String name);

    String getDescription();

    void setDescription(String description);

    Integer getDescriptionHeight();

    void setDescriptionHeight(Integer integer);

    Boolean getDescriptionCollapsed();

    void setDescriptionCollapsed(Boolean collapsed);

    String getId();

    void setId(String id);

    String getLastModified();

    void setLastModified(String modified);

    EntityTo getParent();

    void setParent(EntityTo parent);

    boolean isShared();

    void setShared(boolean shared);

    boolean isLocked();

    void setLocked(boolean isLocked);

    String getRemoteId();

    void setRemoteId(String remoteId);

    String getRemoteModification();

    void setRemoteModification(String token);

    boolean isModifiedLocally();

    void setModifiedLocally(boolean modified);

    enum Type {
        Request(HttpRequestTo.class, false),
        Project(ProjectTo.class, false),
        Service(ServiceTo.class, false),
        Context(EnvironmentTo.class, false),
        Scenario(ScenarioTo.class, false),
        RemoteUserDrive(RemoteUserDriveTo.class, true),
        LocalDriveRoot(null, true),
        SharedDriveRoot(null, true);

        final Class<? extends EntityTo> typeClass;
        public final boolean isDrive;

        Type(final Class<? extends EntityTo> typeClass, final boolean isDrive) {
            this.typeClass = typeClass;
            this.isDrive = isDrive;
        }

        public static RepositoryElementTypeComparator getEntityTypeComparator() {
            return new RepositoryElementTypeComparator();
        }

        public Class<? extends EntityTo> typeClass() {
            return this.typeClass;
        }

        public boolean is(final EntityTo entityTo) {
            return Objects.equals(this, entityTo.getType());
        }

        public boolean isGreaterThan(final Type otherType) {
            return getEntityTypeComparator().compare(this, otherType) > 0;
        }

        public boolean isLowerThan(final Type otherType) {
            return getEntityTypeComparator().compare(this, otherType) < 0;
        }

        public boolean isGreaterOrEquals(final Type otherType) {
            return getEntityTypeComparator().compare(this, otherType) >= 0;
        }

        public boolean isLowerOrEquals(final Type otherType) {
            return getEntityTypeComparator().compare(this, otherType) <= 0;
        }

        public static class RepositoryElementTypeComparator implements Comparator<Type> {
            @Override
            public int compare(final Type type1, final Type type2) {
                final List<Type> order = asList(RemoteUserDrive, Project, Service, Scenario, Request);

                final int type1Index = order.indexOf(type1);
                final int type2Index = order.indexOf(type2);

                if (type1Index == -1 || type2Index == -1) {
                    throw new IllegalArgumentException("Compared elements must be repository elements. Got: " + type1 + " and " + type2);
                }

                return -Integer.compare(type1Index, type2Index);
            }
        }
    }

}
