package com.restlet.client.model.impl;

import com.restlet.client.net.http.HttpExampleTo;
import com.restlet.client.net.http.request.HttpMethodDefinition;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.net.request.RequestBodyTo;
import com.restlet.client.net.request.RequestHeaderTo;
import com.restlet.client.net.request.RequestHeadersType;
import com.restlet.client.net.uri.UriTo;
import com.restlet.client.tests.asserts.AssertionTo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class HttpRequestToImpl extends EntityToImpl implements HttpRequestTo, Serializable {

    private static final long serialVersionUID = 1961013222667388397L;

    private HttpMethodDefinition method;
    private List<RequestHeaderTo> headers = new ArrayList<>();
    private RequestHeadersType headersType;
    private boolean uriEditor;
    private RequestBodyTo body;
    private List<HttpExampleTo> examples = new ArrayList<>();
    private UriTo uri;
    private List<AssertionTo> assertions = new ArrayList<>();
    private Map<String, String> metaInfo = new HashMap<>();
    private boolean bodyEvaluationDisabled = false;

    public HttpRequestToImpl() {
        setType(Type.Request);
    }

    @Override
    public HttpMethodDefinition getMethod() {
        return method;
    }

    @Override
    public void setMethod(final HttpMethodDefinition method) {
        this.method = method;
    }

    @Override
    public List<RequestHeaderTo> getHeaders() {
        return headers;
    }

    @Override
    public void setHeaders(final List<RequestHeaderTo> headers) {
        this.headers = headers;
    }

    @Override
    public RequestHeadersType getHeadersType() {
        return headersType;
    }

    @Override
    public void setHeadersType(final RequestHeadersType headersType) {
        this.headersType = headersType;
    }

    @Override
    public boolean getUriEditor() {
        return uriEditor;
    }

    @Override
    public void setUriEditor(final boolean uriEditor) {
        this.uriEditor = uriEditor;
    }

    @Override
    public RequestBodyTo getBody() {
        return body;
    }

    @Override
    public void setBody(final RequestBodyTo body) {
        this.body = body;
    }

    @Override
    public List<HttpExampleTo> getExamples() {
        return examples;
    }

    @Override
    public void setExamples(final List<HttpExampleTo> examples) {
        this.examples = examples;
    }

    @Override
    public Map<String, String> getMetaInfo() {
        return this.metaInfo;
    }

    @Override
    public void setMetaInfo(final Map<String, String> meta) {
        this.metaInfo = meta;
    }

    @Override
    public boolean isBodyEvaluationDisabled() {
        return bodyEvaluationDisabled;
    }

    @Override
    public void setBodyEvaluationDisabled(final boolean value) {
        bodyEvaluationDisabled = value;
    }

    @Override
    public UriTo getUri() {
        return uri;
    }

    @Override
    public void setUri(final UriTo uri) {
        this.uri = uri;
    }

    @Override
    public boolean equals(final HttpRequestTo requestDto) {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public String getShareLink() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void setShareLink(final String link) {
        // TODO Auto-generated method stub

    }

    @Override
    public List<AssertionTo> getAssertions() {
        return assertions;
    }

    @Override
    public void setAssertions(final List<AssertionTo> assertions) {
        this.assertions = assertions;
    }

    @Override
    public boolean containsNoErrors() {
        return true;
    }

    @Override
    public void wrap(final HttpRequestTo request) {
        // TODO Auto-generated method stub
    }

    @Override
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;

        final HttpRequestToImpl that = (HttpRequestToImpl) o;

        if (uriEditor != that.uriEditor) return false;
        if (bodyEvaluationDisabled != that.bodyEvaluationDisabled) return false;
        if (method != null ? !method.equals(that.method) : that.method != null) return false;
        if (headers != null ? !headers.equals(that.headers) : that.headers != null) return false;
        if (headersType != that.headersType) return false;
        if (body != null ? !body.equals(that.body) : that.body != null) return false;
        if (examples != null ? !examples.equals(that.examples) : that.examples != null) return false;
        if (uri != null ? !uri.equals(that.uri) : that.uri != null) return false;
        if (assertions != null ? !assertions.equals(that.assertions) : that.assertions != null) return false;
        return metaInfo != null ? metaInfo.equals(that.metaInfo) : that.metaInfo == null;

    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + (method != null ? method.hashCode() : 0);
        result = 31 * result + (headers != null ? headers.hashCode() : 0);
        result = 31 * result + (headersType != null ? headersType.hashCode() : 0);
        result = 31 * result + (uriEditor ? 1 : 0);
        result = 31 * result + (body != null ? body.hashCode() : 0);
        result = 31 * result + (examples != null ? examples.hashCode() : 0);
        result = 31 * result + (uri != null ? uri.hashCode() : 0);
        result = 31 * result + (assertions != null ? assertions.hashCode() : 0);
        result = 31 * result + (metaInfo != null ? metaInfo.hashCode() : 0);
        result = 31 * result + (bodyEvaluationDisabled ? 1 : 0);
        return result;
    }
}
