package com.restlet.client.platform.crypto;

import com.restlet.client.platform.CryptoApi;

public abstract class AbstractCryptoImpl implements CryptoApi {

    @Override
    public String hash(HashType hashType, String message, Output out) {
        String hash = (hashType == null) ? null : hashType.algorithm;
        String output = (out == null) ? null : out.label;

        return getHash(message, hash, output);
    }

    @Override
    public String hmac(HashType hashType, String secret, String message, Output out) {
        String hash = (hashType == null) ? null : hashType.algorithm;
        String output = (out == null) ? null : out.label;

        return getHMAC(secret, message, hash, output);
    }

    @Override
    public String encrypt(CipherType type, String message, String secret) {
        if (type == null) {
            throw new IllegalArgumentException("The argument cipher type is missing");
        }
        switch (type) {
            case AES:
                return encryptAES(message, secret);
        }
        throw new IllegalStateException("The cipher type '" + type + "' is not supported.");
    }

    @Override
    public String decrypt(CipherType type, String message, String secret) {
        if (type == null) {
            throw new IllegalArgumentException("The argument cipher type is missing");
        }
        switch (type) {
            case AES:
                return decryptAES(message, secret);
        }
        throw new IllegalStateException("The cipher type '" + type + "' is not supported.");
    }

    protected abstract String encryptAES(String message, String secret);

    protected abstract String decryptAES(String message, String secret);

    protected abstract String getHMAC(String secret, String message, String hash, String output);

    protected abstract String getHash(String message, String hash, String output);
}
