package com.restlet.client.script.value;

import com.restlet.client.async.Promise;
import com.restlet.client.async.Promises;
import com.restlet.client.platform.json.JsonEngine;
import com.restlet.client.platform.json.JsonException;
import com.restlet.client.platform.json.JsonValue;
import com.restlet.client.script.runtime.ScriptValue;
import com.restlet.client.utils.StringUtils;

public class ScriptStringValue implements ScriptValue {

    private final String value;

    private final JsonEngine jsonParser;

    public ScriptStringValue(String value, JsonEngine jsonParser) {
        super();
        this.value = value;
        this.jsonParser = jsonParser;
    }

    @Override
    public String toString() {
        return value;
    }

    @Override
    public Promise<ScriptValue> select(String path) {
        if (StringUtils.isBlank(value)) {
            return Promises.of();
        }

        try {
            JsonValue json = jsonParser.fromJson(value);
            if (json != null) {
                if (json.isObject() != null) {
                    JsonValue jsonValue = json.isObject().get(path);
                    if (jsonValue != null) {
                        return Promises.of((ScriptValue) new ScriptJsonValue(jsonValue, jsonParser));
                    }
                } else if (json.isArray() != null) {
                    try {
                        Integer index = Integer.valueOf(path);
                        if (index != null && index >= 0 && index < json.isArray().size()) {
                            JsonValue jsonValue = json.isArray().get(index);
                            if (jsonValue != null) {
                                return Promises.of((ScriptValue) new ScriptJsonValue(jsonValue, jsonParser));
                            }
                        }

                    } catch (NumberFormatException e) {
//                        do nothing
                    }
                }
            }

        } catch (JsonException e) {
            e.printStackTrace();
        }

        return Promises.of();
    }
}
