package com.restlet.client.tests.asserts;

import java.util.Collection;
import java.util.Comparator;
import java.util.LinkedHashSet;
import java.util.List;

import static java.util.Arrays.asList;

public class AssertionResultTo implements AssertionTo {

    public static final String ASSERTION_PATH_STATUS_MESSAGE = "message";
    public static final String ASSERTION_PATH_STATUS_CODE = "code";
    public static final String ASSERTION_PATH_RESPONSE_BODY_CONTENT = "content";
    public static final String ASSERTION_PATH_RESPONSE_BODY_LENGTH = "length";
    public static final String ASSERTION_PATH_RESPONSE_LATENCY = "latency";

    private State state;
    private final AssertionTo assertion;
    private final Collection<AssertionResultMessage> messages = new LinkedHashSet<>();
    private String evaluatedPath;
    private String evaluatedValue;
    private String actualValue;

    public AssertionResultTo(final AssertionTo assertion) {
        this.assertion = assertion;
    }

    @Override
    public Boolean isEnabled() {
        return assertion.isEnabled();
    }

    @Override
    public void setEnabled(final Boolean enabled) {
        assertion.setEnabled(enabled);
    }

    @Override
    public Subject getSubject() {
        return assertion.getSubject();
    }

    @Override
    public void setSubject(final Subject subject) {
        assertion.setSubject(subject);
    }

    @Override
    public String getPath() {
        return assertion.getPath();
    }

    @Override
    public void setPath(final String path) {
        assertion.setPath(path);
    }

    @Override
    public Comparison getComparison() {
        return assertion.getComparison();
    }

    @Override
    public void setComparison(final Comparison comparison) {
        assertion.setComparison(comparison);
    }

    @Override
    public String getValue() {
        return assertion.getValue();
    }

    @Override
    public void setValue(final String value) {
        assertion.setValue(value);
    }

    public final AssertionTo getAssertion() {
        return assertion;
    }

    public State getState() {
        return state;
    }

    public void setState(final State state) {
        this.state = state;
    }

    public Collection<AssertionResultMessage> getMessages() {
        return messages;
    }

    public Collection<AssertionResultMessage> getMessages(final State state) {
        final Collection<AssertionResultMessage> messagesByState = new LinkedHashSet<>();
        for (final AssertionResultMessage message : messages) {
            if (message.state == state) {
                messagesByState.add(message);
            }
        }
        return messagesByState;
    }

    public void addFailureMessage(final String message) {
        messages.add(new AssertionResultMessage(State.Failure, message));
    }

    public void addErrorMessage(final String message) {
        messages.add(new AssertionResultMessage(State.Error, message));
    }

    public void addWarningMessage(final String message) {
        messages.add(new AssertionResultMessage(State.Warning, message));
    }

    public String getEvaluatedPath() {
        return evaluatedPath;
    }

    public void setEvaluatedPath(final String evaluatedPath) {
        this.evaluatedPath = evaluatedPath;
    }

    public String getEvaluatedValue() {
        return evaluatedValue;
    }

    public void setEvaluatedValue(final String evaluatedValue) {
        this.evaluatedValue = evaluatedValue;
    }

    public String getActualValue() {
        return actualValue;
    }

    public AssertionResultTo setActualValue(final String actualValue) {
        this.actualValue = actualValue;
        return this;
    }

    public enum State {
        Ok, Failure, Error, Warning;

        private static final List<State> ORDERED_STATES = asList(Error, Failure, Warning, Ok);

        public static Comparator<State> getStateComparator() {
            return new Comparator<State>() {
                @Override
                public int compare(final State state1, final State state2) {
                    final Integer state1Index = ORDERED_STATES.indexOf(state1);
                    if (state1Index == -1) {
                        throw new RuntimeException("State " + state1 + " is not in the ordered list of states.");
                    }

                    final Integer state2Index = ORDERED_STATES.indexOf(state2);
                    if (state2Index == -1) {
                        throw new RuntimeException("State " + state2 + " is not in the ordered list of states.");
                    }

                    return -state1Index.compareTo(state2Index);
                }
            };
        }
    }

    public static class AssertionResultMessage {
        public State state;
        public String message;

        AssertionResultMessage(final State state, final String message) {
            this.state = state;
            this.message = message;
        }

        @Override
        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final AssertionResultMessage that = (AssertionResultMessage) o;

            if (state != that.state) return false;
            return message != null ? message.equals(that.message) : that.message == null;
        }

        @Override
        public int hashCode() {
            int result = state != null ? state.hashCode() : 0;
            result = 31 * result + (message != null ? message.hashCode() : 0);
            return result;
        }
    }
}
