package com.restlet.client.utils;

import com.restlet.client.function.BiFunction;
import com.restlet.client.function.Function;
import com.restlet.client.model.EntityTo;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.tests.ScenarioTo;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import static com.restlet.client.utils.EntityUtils.indexOf;

public class ComparatorUtils {

    public static void sortHttpRequests(final ScenarioTo scenarioTo) {
        if (scenarioTo == null) {
            return;
        }
        sortAsScenarioRequests(scenarioTo.getHttpRequests(), scenarioTo, new Function<HttpRequestTo, String>() {
            @Override
            public String apply(final HttpRequestTo requestTo) {
                return requestTo.getName();
            }
        });
    }

    /**
     * According to the scenario requests order, sorts list where elements as similar to HttpRequestTo instances.
     * At least each element is able to compute the name of one of the scenario's HTTP request.
     */
    public static <T> void sortAsScenarioRequests(final List<T> toSort, final ScenarioTo scenarioTo, final Function<T, String> nameMapper) {
        if (scenarioTo == null || toSort == null) {
            return;
        }
        Collections.sort(toSort, new Comparator<T>() {
            @Override
            public int compare(final T o1, final T o2) {
                final int index1 = indexOf(scenarioTo, nameMapper.apply(o1));
                final int index2 = indexOf(scenarioTo, nameMapper.apply(o2));

                return index1 - index2;
            }
        });
    }

    public static BiFunction<EntityTo, EntityTo, Boolean> equalityMatcherById = new BiFunction<EntityTo, EntityTo, Boolean>() {
        @Override
        public Boolean apply(final EntityTo e1, final EntityTo e2) {
            return java.util.Objects.equals(e1.getId(), e2.getId());
        }
    };
}
