package com.restlet.client.utils;

import com.restlet.client.function.Consumer;
import com.restlet.client.function.Executor;
import com.restlet.client.function.Function;
import com.restlet.client.function.Supplier;

public class FunctionalUtils {

    public static <R, T> T ifNotNull(final R object, final Function<R, T> getter) {
        return ifNotNull(object, getter, null);
    }

    public static <R, T> T ifNotNull(final R object, final Function<R, T> getter, final T fallback) {
        return object == null ? fallback : getter.apply(object);
    }

    public static <T> Function<T, T> identity() {
        return new Function<T, T>() {
            @Override
            public T apply(final T input) {
                return input;
            }
        };
    }

    public static <T> void doIfNotNull(final T object, final Consumer<T> successHandler) {
        doIfNotNull(object, successHandler, noopExecutor());
    }

    public static <T> void doIfNotNull(final T object, final Consumer<T> successHandler, final Executor failHandler) {
        doIf(object != null,
                new Executor() {
                    @Override
                    public void execute() {
                        successHandler.consume(object);
                    }
                }, failHandler);
    }

    public static void doIf(final boolean condition, final Executor successExecutor) {
        doIf(condition, successExecutor, new Executor() {
            @Override
            public void execute() {}
        });
    }

    public static void doIf(final boolean condition, final Executor successExecutor, final Executor failureExecutor) {
        if (condition) {
            successExecutor.execute();
        } else {
            failureExecutor.execute();
        }
    }

    public static <T> Consumer<T> noopConsumer() {
        return new Consumer<T>() {
            @Override
            public void consume(final T value) {}
        };
    }

    public static Executor noopExecutor() {
        return new Executor() {
            @Override
            public void execute() {}
        };
    }

    public static <T> T returnIf(final boolean condition, final Supplier<T> successSupplier, final Supplier<T> failureSupplier) {
        if (condition) {
            return successSupplier.get();
        } else {
            return failureSupplier.get();
        }
    }

}
