package com.restlet.client.utils;

import com.restlet.client.async.Promise;
import com.restlet.client.async.Promises;
import com.restlet.client.dao.RepositoryDao;
import com.restlet.client.function.BiConsumer;
import com.restlet.client.function.Function;
import com.restlet.client.model.EntityTo;
import com.restlet.client.model.EntityTreeNode;

import java.util.List;

import static com.restlet.client.model.UpdateType.Import;

public class RepositoryUtils {

    public static Promise<EntityTreeNode> readEntityTree(final RepositoryDao repositoryDao, final EntityTo wrapperEntity,
                                                         final List<String> rootEntityNames) {
        final EntityTreeNode entityTreeNode = new EntityTreeNode(wrapperEntity, Import);
        return Promises.all(Sequence.of(rootEntityNames)
                .map(new Function<String, Promise<EntityTreeNode>>() {
                    @Override
                    public Promise<EntityTreeNode> apply(final String name) {
                        return repositoryDao.find(name, null)
                                .flatMap(new Function<EntityTo, Promise<EntityTreeNode>>() {
                                    @Override
                                    public Promise<EntityTreeNode> apply(final EntityTo persistedRootEntity) {
                                        return persistedRootEntity == null
                                                ? null
                                                : repositoryDao.readEntityTree(persistedRootEntity.getId());
                                    }
                                });
                    }
                })
                .toList())
                .map(new Function<List<EntityTreeNode>, EntityTreeNode>() {
                    @Override
                    public EntityTreeNode apply(final List<EntityTreeNode> children) {
                        Sequence.of(children)
                                .omitNulls()
                                .each(new BiConsumer<EntityTreeNode, Integer>() {
                                    @Override
                                    public void consume(final EntityTreeNode child, final Integer secondValue) {
                                        entityTreeNode.children.add(child);
                                    }
                                });
                        return entityTreeNode;
                    }
                });
    }
}
